
#ifndef ALIGNER
#define ALIGNER

#include "CBT.hh"
#include "Table.hh"
//#include "Sequence3Bit.hh"

//#include "Subsequence.hh"

typedef Sequence Subsequence;

//typedef Sequence3Bit Subsequence;
/*
  Matrix used by crossmatch.  
  Score matrix (set by value of penalty: -2)
      A   C   G   T   N   X
  A   1  -2  -2  -2   0  -3
  C  -2   1  -2  -2   0  -3
  G  -2  -2   1  -2   0  -3
  T  -2  -2  -2   1   0  -3
  N   0   0   0   0   0   0
  X  -3  -3  -3  -3   0  -3

*/

inline  void swap(int &a,int &b)
{
  int tmp = a;
  a = b;
  b = tmp;
}

//----------------------------------------------------------------------------------
// * Int2D
//----------------------------------------------------------------------------------
// An attempt to exert more control over performance than I can with Table 
//
class Int2D{
 public:  
  int **pData;

  int mMaxCols,mMaxRows;

  inline int* operator[](int n) { return pData[n];};
  inline const int* operator[](int n) const { return pData[n]; }

  void Allocate(int maxcol,int maxrows){
    mMaxCols = maxcol;
    mMaxRows = maxrows;
    pData = new int*[maxcol];
    for(int i = 0;i < maxcol;i++){
      pData[i] = new int[maxrows];

      // Clear everything for starters..
      for(int j = 0;j < maxrows;j++){
	pData[i][j] = 0;
      }

    }
  }

  void DeAllocate(){
    for(int i = 0;i<mMaxCols;i++){
      delete [] pData[i];
    }
    delete [] pData;
  }

  void Init(int value){
    for(int i = 0;i < mMaxCols;i++){
      for(int j = 0;j < mMaxRows;j++){
	pData[i][j] = value;
      }
    }
  }
};




//----------------------------------------------------------------------------------
// * Substitution
//----------------------------------------------------------------------------------
//
// The default substitution matrix just has a fixed 
// match or mismatch score.  It can be overridden by 
// more complex matrices such as Blosum50 or whatever. 
class Substitution{
 public:

  int mMatchScore;
  int mMisMatchScore;
  
  virtual int score(char a,char b) = 0;
  virtual int normalScore(char a,char b) = 0;
  virtual int operator()(char a,char b) = 0;

};




//----------------------------------------------------------------------------------
// * DefaultSubstitution
//----------------------------------------------------------------------------------
//
// The default substitution matrix just has a fixed 
// match or mismatch score.  It can be overridden by 
// more complex matrices such as Blosum50 or whatever. 
class DefaultSubstitution : public Substitution{
 public:

  int mMatchScore;
  int mMisMatchScore;
  
  DefaultSubstitution(){
    mMatchScore = 1;
    mMisMatchScore = -1;
  }

  inline int score(char a,char b){
    if (a == b) return(mMatchScore);
    else return(mMisMatchScore);
  }


  inline int normalScore(char a,char b){
    if (a == b) return(mMatchScore);
    else return(mMisMatchScore);
  }

  inline int operator()(char a,char b){
    //    if ((a == 'n') || (b =='n')) return(0);
    if (a == b) return(mMatchScore);
    else return(mMisMatchScore);
  }

};


//----------------------------------------------------------------------------------
// * Traceback3
//----------------------------------------------------------------------------------
//
class Traceback3{
 public:
  int k,i, j;

  Traceback3(){};

  Traceback3 & operator()(int c, int a,int b){
    k = c;
    i = a;
    j = b;
    return(*this);
  }

  Traceback3(int c, int a,int b){
    k = c;
    i = a;
    j = b;
  }

  void Print(ostream &fout){
    fout<<"("<<k<<","<<i<<","<<j<<")";
  }

  //  bool operator==(Traceback &a,Traceback &b){
  // return((a.i == b.i) && (a.j == b.j));
  // }
};

//----------------------------------------------------------------------------------
// * Traceback
//----------------------------------------------------------------------------------
//
class Traceback{
 public:
  int i, j;

  Traceback(){};

  Traceback & operator()(int a,int b){
    i = a;
    j = b;
    return(*this);
  }

  Traceback(int a,int b){
    i = a;
    j = b;
  }

  void Print(ostream &fout){
    fout<<"("<<i<<","<<j<<")";
  }

  //  bool operator==(Traceback &a,Traceback &b){
  // return((a.i == b.i) && (a.j == b.j));
  // }
};

//----------------------------------------------------------------------------------
// * Aligner
//----------------------------------------------------------------------------------
// 
class Aligner {
 public: 
  int                 mGapCost;
  Subsequence         mSeq1,mSeq2;
  int                 mN,mM;
  Traceback           mB0;
  
  int mStart1, mStart2;
  int mEnd1,mEnd2;

  Substitution  *mpSub;

  static const int NegInf = -2147483647; // -(2^32 -1)

  Aligner(){};

  Aligner(Substitution *ps,int GapCost,Subsequence &s1,Subsequence& s2){
    mpSub = ps;
    mGapCost = GapCost;
    mSeq1 = s1;
    mSeq2 = s2;	 

    // Case must match. 
    //    mSeq1.ToLowerCase();
    //mSeq2.ToLowerCase();

    mN = s1.size();
    mM = s2.size();
  }

  void getMatch(Subsequence &res1,Subsequence &res2);

  virtual   Traceback next(Traceback &tb) =0;

  //  Traceback next(Traceback &tb){
  // }
  //  int nextI(int i,int j) = 0;
  //int nextJ(int i,int j) = 0;

  int getScore(){
    return(-1);
  }
  
  static int max(int x1,int x2)
  {
    return(x1 > x2 ? x1 : x2);
  }

  static int max(int x1,int x2,int x3)
  {
    return(max(x1,max(x2,x3)));
  }

  static int max(int x1,int x2,int x3,int x4)
  {
    return(max(max(x1,x2),max(x3,x4)));
  }

  static int max(int x1,int x2,int x3,int x4,int x5)
  {
    return( max(max(x1,x2),max(x3,x4),x5) );
  }

  static int max(int x1,int x2,int x3,int x4,int x5,int x6)
  {
    return(max(max(x1,x2),max(x3,x4),max(x5,x6)) );
  }

  static int max(int x1,int x2,int x3,int x4,int x5,int x6,int x7)
  {
    return(max(max(x1,x2),max(x3,x4),max(x5,x6),x7) );
  }

};

//----------------------------------------------------------------------------------
// * BasicAligner
//----------------------------------------------------------------------------------
// 
class BasicAligner : public Aligner{
 public:

  Table<int> F;           // Alignment matrix
  Table<Traceback> B;     // Traceback matrix. 

  BasicAligner(Substitution *psub, int GapCost,Subsequence &s1,Subsequence &s2)
    :Aligner(psub,GapCost,s1,s2)
  {
    F.Allocate(mN+1,mM+1);
    B.Allocate(mN+1,mM+1);

    // Initially mark entire traceback table with stop symbol. 
    Traceback tb(-1,-1);
    B.Init(tb);   
  }

  Traceback next(Traceback &tb){
    return(B[tb.i][tb.j]);
  }

  int getScore(){
    return(F[mB0.i][mB0.j]);
  }
};


//----------------------------------------------------------------------------------
// * NWAligner
//----------------------------------------------------------------------------------
// Global alignment. Needleman-Wunsch. 
// 
class NWAligner : public BasicAligner{
 public:
  NWAligner(Substitution *psub, int GapCost,Subsequence &s1,Subsequence &s2);
};



//----------------------------------------------------------------------------------
// * SWAligner
//----------------------------------------------------------------------------------
// Local alignment. Smith-Waterman
// 
class SWAligner : public BasicAligner{
 public:
  SWAligner(Substitution *psub, int GapCost,Subsequence &s1,Subsequence &s2);


  int getSourceCoords(int &start,int &end){
    start = mStart1;
    end = mEnd1-1;
  }

  int getQueryCoords(int &start,int &end){
    start = mStart2;
    end = mEnd2;
  }
};




//----------------------------------------------------------------------------------
// * SWAligner3
//----------------------------------------------------------------------------------
// Local alignment. Smith-Waterman
// 
class SWAligner3 {
 public:
  
  int                 mGapCost;
  Subsequence         mSeq1,mSeq2,mSeq3;
  int                 mN,mM;  
  Substitution  *mpSub;

  static const int NegInf = -2147483647; // -(2^32 -1)
  
  Traceback3 mB0;
  vector<Table<int> >F3;           // Alignment matrix
  vector<Table<Traceback3> > B3;     // Traceback matrix. 
  
  SWAligner3(Substitution *psub, int GapCost,Subsequence &s1,Subsequence &s2,Subsequence &s3);

  void getMatch(Subsequence &res1,Subsequence &res2);

  void InitTraceback(Traceback3 &tb){
    B3[0].Init(tb);
    B3[1].Init(tb);
  }

  Traceback3 next(Traceback3 &tb){
    return(B3[tb.k][tb.i][tb.j]);
  }

  int getScore(){
    return(F3[mB0.k][mB0.i][mB0.j]);
  }
  
  static int max(int x1,int x2)
  {
    return(x1 > x2 ? x1 : x2);
  }

  static int max(int x1,int x2,int x3)
  {
    return(max(x1,max(x2,x3)));
  }

  static int max(int x1,int x2,int x3,int x4)
  {
    return(max(max(x1,x2),max(x3,x4)));
  }

  static int max(int x1,int x2,int x3,int x4,int x5)
  {
    return( max(max(x1,x2),max(x3,x4),x5) );
  }

  static int max(int x1,int x2,int x3,int x4,int x5,int x6)
  {
    return(max(max(x1,x2),max(x3,x4),max(x5,x6)) );
  }

  static int max(int x1,int x2,int x3,int x4,int x5,int x6,int x7)
  {
    return(max(max(x1,x2),max(x3,x4),max(x5,x6),x7) );
  }
};




//----------------------------------------------------------------------------------
// * LinearAligner
//----------------------------------------------------------------------------------
// 
class LinearAligner : public Aligner{
 public:

  //Table<int> F;           // Alignment matrix
  Int2D F;

  LinearAligner(Substitution *psub, int GapCost,Subsequence &s1,Subsequence &s2)
    :Aligner(psub,GapCost,s1,s2){

      F.Allocate(2,mM+1);
    }

  /*
  void swap(int &a,int &b)
  {
    int tmp = a;
    a = b;
    b = tmp;
  }
  */


  // Dummy function.  This seems messy. Why bother 
  // with making this a part of Aligner???
  Traceback next(Traceback &tb){
    return(tb);
  }

};


//----------------------------------------------------------------------------------
// * NWLinearAligner
//----------------------------------------------------------------------------------
// Linear space global alignment. 
// 
class NWLinearAligner : public LinearAligner{
 public:
  
  int mU;
  //  Table<int> C;
  Int2D C;
  int mOldCol,mNewCol;

  NWLinearAligner(Substitution *psub, int GapCost,Subsequence &s1,Subsequence &s2);
  
  int getV(){
    int v =C[mNewCol][mM]; 
    //cout<<"getV["<<mNewCol<<"]["<<mM<<"]="<<v<<endl;
    return(v);
  }
	
  void getMatch(Subsequence &r1,Subsequence &r2);

  void ComputeScore(Subsequence &s1,Subsequence &s2);

  int getScore(){
    return(F[mNewCol][mM]);
  }
};

/*

class TraceBackTable{
  int *pI;
  int *pJ;

  Allocate(

}


  int n = mN, m = mM;
  start.Allocate(2,m+1);
  Traceback tb;
  start.Init(tb(-1,-1));
  mMaxVal = NegInf;

  int newcol = 1, oldcol = 0;
  for(int j = 0;j<=m;j++)
    start[newcol][j] = tb(0,j);

*/

//----------------------------------------------------------------------------------
// * SWLinearAligner
//----------------------------------------------------------------------------------
// Linear space global alignment. 
// 
class SWLinearAligner : public LinearAligner{
 public:
  
  //Table<Traceback> start;
  Int2D startI;
  Int2D startJ;

  int mMaxVal;
  int mStart1, mStart2;
  int mEnd1,mEnd2;

  int mOldCol,mNewCol;

  SWLinearAligner(Substitution *psub, int GapCost,Subsequence &s1,Subsequence &s2);
  	
  void getMatch(Subsequence &r1,Subsequence &r2);

  void ComputeScore(Subsequence &s1,Subsequence &s2);
  int getScore(){
    return(mMaxVal);
  }

  int getSourceCoords(int &start,int &end){
    start = mStart1;
    end = mEnd1-1;
  }

  int getQueryCoords(int &start,int &end){
    start = mStart2;
    end = mEnd2-1;
  }

};


#endif 



























