#ifndef HITCHAINER
#define HITCHAINER

#include "CBT.hh"
#include "SearchResult.hh"

class Segment{
public:
  long mScore;
  long mSBegin;
  long mSEnd;
  long mQBegin;
  long mQEnd;
};

// A chain is made up of a list of segments.  Each 
// segment is an span of exactly matching elements. 
// A chain may contain mismatches, gaps, or whatever, 
// but only between the segments.   The simplest kind of 
// segment is simply a sample.  
class Chain : public vector<Segment>{
public:  

  Chain(){
    mScore = 0;
    mSourceName = "";
    mQueryName = "";
    mSBegin = 0;
    mSEnd = 0;
    mQBegin = 0;
    mQEnd = 0;
    mAvgCount = 0;
    mTotalBases = 0;
    
    mSourceSize = 0;
    mQuerySize = 0;
  }

  long mScore;

  string mSourceName;
  string mQueryName;

  long mQuerySize;
  long mSourceSize;

  long mSBegin;
  long mSEnd;
  long mQBegin;
  long mQEnd;

  float mAvgCount;
  int   mTotalBases;

  void Print(ostream &fout){
    cout<<"Need to resolve how this will know source name."<<endl;
    //    fout<<mSourceName<<"\t"<<mQueryName<<"\t";
    //fout<<mScore<<"\t"<<mSBegin<<"\t"<<mSEnd<<"\t";
    //fout<<mQBegin<<"\t"<<mQEnd<<endl;
  }
};

class HitChainer{
 public:

  HitChainer(){
    //    f = 7;
    ///slice = 0; 
    mWinLength = 500;
    mWinWidth = 100;
    mMinScore = 50;
  }

  HitChainer(int winlen,int winwidth){
    //f = 7;
    //slice = 0; 
    mWinLength = winlen;
    mWinWidth = winwidth;
    mMinScore = 50;
  }

  string mSourceName;
  string mQueryName;
  int   mQuerySkip;
  int   mSourceSkip;
  int   mWinLength;
  int   mWinWidth;
  int   mMinScore;
  int   mX0;
  int   mY0;
  bool  mbTabbedOut;
  //  long long f;
  //long long slice;
  

  string mSettingsKey;

  string getSettingsKey(){return(mSettingsKey);};


  void ComputeChains(SearchResultVec &Hits, vector<Chain> &Chains);
  void ComputeChains0(SearchResultVec &Hits, vector<Chain> &Chains); // Old version. 
  
  void WriteLV(ostream &fout,Sequence &Query,Chain &theChain);
  void WriteLVHeader(ostream &fout){
    fout<<"#:lav"<<endl;
  }
  void WriteLVFooter(ostream &fout){
    fout<<"#:eof"<<endl;  
  }

  void WriteTabbedHeader(ostream &fout);
  void WriteTabbed(ostream &fout,Sequence &Query,Chain &theChain,bool bIsReverse);
  void WriteTabbed(ostream &fout,Sequence &Query,vector<Chain> &chains,bool bIsReverse){
    for(int i = 0;i < chains.size();i++) {
      //cout<<"Writing chain#"<<i<<endl;
      WriteTabbed(fout,Query,chains[i],bIsReverse);
    }
  }

  void ReadTabbed(istream &fin,vector<Chain> &C);
  
  bool InWindow(Chain &MyChain,SearchResult &SR);

};

#endif 
