
#ifndef MER2INTMAP
#define MER2INTMAP
#include "CBT.hh"
#include "Mer.hh"
#include "MerHashF.hh"

#define SEQ_BUFFER_SIZE2 50000
extern char gseqbuf2[SEQ_BUFFER_SIZE2];

//----------------------------------------------------------------------------------------
// * Mer2IntMap
//----------------------------------------------------------------------------------------
// A simple mer hash map that saves only the count of the mer. 
// 
// Obviously, we could also define an Int class and then instantiate 
// MerHashMap<Int> to get the same effect.   This is a common enough thing to do that 
// it's implemented here as it's own class, both as syntatic sugar and to save the 
// small extra cost of having an Int class to dereference.  
// 
// 
template<class MerHash> 
#ifdef ICC
class Mer2IntMap: public hash_map<Mer,int,MerHash,allocator<Mer> >{
#else
  class Mer2IntMap: public hash_map<Mer,int,MerHash,MerHash>{
#endif 
    
  public:
    
    MerHash MH;
    
    Mer2IntMap(){
      mTotalMersInHash = 0;
    }

    // KJD This is all only stored in Mer:: now. 
    //void setThinFactor(uint32 f){
    //  mThinFactor = f;
    // }
    
  //void setThinSlice(uint32 s){
  //  mThinSlice = s;
  // }

  //uint32 mThinFactor;
  //uint32 mThinSlice;

  void Add(string &FName){
    izipstream zin;
    zin.open(FName.c_str());
    Add(zin);
    zin.close();
  }

  void Add(istream &inFile);
  void Add(Sequence &Seq);
  void Add(Sequence3Bit &Seq);
  void Add(Sequence3Bit &Seq,Mer2IntMap<MerHash> &FilterSet);

  void Add(Mer &TempMer);

  int mTotalMersInHash;  
  vector<string> mNames;

  bool Contains(Mer M){
    typename Mer2IntMap<MerHash>::iterator it;
    it = (*this).find(M);
		  
    if (it != (*this).end()) return(true);
    else return(false);
  }

  bool Contains(Mer M,int &count){
    typename Mer2IntMap::iterator it;
    it = (*this).find(M);    
    count = 0;
    if (it != (*this).end()) {
      count = (*it).second;
      return(true);
    }else return(false);
  }
  
};
  
// I can't seem to make this work in the .cc file... ??



//---------------------------------
// * Add
//---------------------------------
// Adds a sequence to the Mer2IntMap counting the occurrences 
// of each mer.  
//
template<class MerHash>
void
Mer2IntMap<MerHash>::Add(Sequence3Bit &Seq)
{
  MerHash MH;
  bool bContainsNX;
  int maxBaseIdx = (Seq.size() - Mer::mMerSize+1);
  for(int baseIdx = 0;baseIdx< maxBaseIdx;baseIdx++){
    // Create a mer out of the next Mer::mMerSize bases starting at baseIdx
    // recording if N or X encountered. 
    Mer TempMer(Seq,baseIdx,bContainsNX);	 

    //    Sequence bugSeq;
    //TempMer.ToSequence(bugSeq);
    if (!bContainsNX){
      if (!TempMer.isThinned()){
	// Record mer...
	typename Mer2IntMap<MerHash>::iterator it;
	it = (*this).find(TempMer);
				
	// If we already know this mer, add to it's count
	if (it != (*this).end()){
	  (*it).second++;
	  mTotalMersInHash++;  // Used to compute a global average. 
	}else{
	  // Otherwise add the mer to the map and set it's count to one. 
	  (*this)[MH.canonicalValue(TempMer)] = 1;
	  mTotalMersInHash++;  // Used to compute a global average. 
	}
	//				cout<<endl;
      }else{
	//cout<<"THIN"<<endl;
      }
    }else{
      //cout<<"CONTAINS NORX"<<endl;
    }
  }
  
  // Record the name of this sequence...
  mNames.push_back(Seq.mName);
}



//---------------------------------
// * Add
//---------------------------------
// Adds a sequence to the Mer2IntMap counting the occurrences 
// of each mer.  This version only adds mers that are found in 
// FilterSet.  All other mers are skipped.  This is used in, for example, 
// the Bayes classifier, where we want to specify a feature subset to use. 
//
template<class MerHash>
void
Mer2IntMap<MerHash>::Add(Sequence3Bit &Seq,Mer2IntMap<MerHash> &FilterSet)
{
  MerHash MH;
  bool bContainsNX;
  int maxBaseIdx = (Seq.size() - Mer::mMerSize+1);
  for(int baseIdx = 0;baseIdx< maxBaseIdx;baseIdx++){
    // Create a mer out of the next Mer::mMerSize bases starting at baseIdx
    // recording if N or X encountered. 
    Mer TempMer(Seq,baseIdx,bContainsNX);	 

    // Skip Mers not in the filter set. 
    if (!FilterSet.Contains(TempMer)) continue;

    //    Sequence bugSeq;
    //TempMer.ToSequence(bugSeq);
    if (!bContainsNX){
      if (!TempMer.isThinned()){
	// Record mer...
	typename Mer2IntMap<MerHash>::iterator it;
	it = (*this).find(TempMer);
				
	// If we already know this mer, add to it's count
	if (it != (*this).end()){
	  (*it).second++;
	  mTotalMersInHash++;  // Used to compute a global average. 
	}else{
	  // Otherwise add the mer to the map and set it's count to one. 
	  (*this)[MH.canonicalValue(TempMer)] = 1;
	  mTotalMersInHash++;  // Used to compute a global average. 
	}
	//				cout<<endl;
      }else{
	//cout<<"THIN"<<endl;
      }
    }else{
      //cout<<"CONTAINS NORX"<<endl;
    }
  }
  
  // Record the name of this sequence...
  mNames.push_back(Seq.mName);
}





template <class MerHash>
void
Mer2IntMap<MerHash>::Add(Mer &TempMer)
{



  //  Mer2IntMap &M = (*this);
  MerHash MH;
  // Do nothing if this mer should be thinned...
  if(TempMer.isThinned()) {
    cout<<"Thinned: "<<TempMer.ToString()<<endl;
    return;
  }
  cout<<"Add "<<TempMer.ToString()<<"\t size="<<(*this).size()<<endl; 
  typename Mer2IntMap<MerHash>::iterator it;
  it = (*this).find(TempMer);				
  // If we already know this mer, add to it's count
  if (it == (*this).end()){
    cout<<"New mer"<<endl;
    cout<<"Canonical value:"<<MH.canonicalValue(TempMer).ToString()<<endl;
    // Otherwise add the mer to the map and set it's count to one. 
    (*this)[MH.canonicalValue(TempMer)] = 1;
    mTotalMersInHash++;  // Used to compute a global average. 
  }else{
    cout<<"Already known."<<endl;
    (*it).second++;
    mTotalMersInHash++;  // Used to compute a global average. 
  }
}



//---------------------------------------------
// * Add
//---------------------------------------------
// 
template<class MerHash>
void
Mer2IntMap<MerHash>::Add(istream &fin)
{
  Sequence tempSeq;
  FAFile FA;    
  
  long readIdx = 0;
  do{
    // KJD BUG This is broken.  Fails to read the last one. 
    gseqbuf2[0] = '1';  // KJD, this is crazy. 

    FA.GetNextSequence(fin,tempSeq);
    //    FA.GetNextSequence(fin,gseqbuf2,SEQ_BUFFER_SIZE2,tempSeq);
    //    tempSeq.GetNextRead(fin,gseqbuf2,SEQ_BUFFER_SIZE2);
    //cout<<"Add: "<<tempSeq<<endl;
    Add(tempSeq);	 	
  }while(!fin.eof());
}


//---------------------------------
// * Add
//---------------------------------
// Adds a sequence to the Mer2IntMap counting the occurrences 
// of each mer.  
//
template<class MerHash>
void
Mer2IntMap<MerHash>::Add(Sequence &Seq)
{
  //  cout<<"ADD***"<<endl;

  MerHash MH;
  bool bContainsNX;
  int maxBaseIdx = (Seq.size() - Mer::mMerSize+1);
  for(int baseIdx = 0;baseIdx< maxBaseIdx;baseIdx++){
    // Create a mer out of the next Mer::mMerSize bases starting at baseIdx
    // recording if N or X encountered. 
    Mer TempMer(Seq,baseIdx,bContainsNX);	
    if (!bContainsNX){
      //cout<<"\t !bContainsNX"<<endl;
      if (!TempMer.isThinned()){
	//cout<<"\t !isThinned"<<endl;
	// Record mer...
	typename Mer2IntMap<MerHash>::iterator it;
	it = (*this).find(TempMer);
				
	// If we already know this mer, add to it's count
	if (it != (*this).end()){
	  //	  cout<<"\t already known."<<endl;
	  (*it).second++;
	  mTotalMersInHash++;  // Used to compute a global average. 
	}else{
	  //cout<<"\t NEW."<<endl;
	  // Otherwise add the mer to the map and set it's count to one. 
	  (*this)[MH.canonicalValue(TempMer)] = 1;
	  mTotalMersInHash++;  // Used to compute a global average. 
	}
	//				cout<<endl;
      }else{
	//	cout<<"\t Thinned"<<endl;
      }
    }else{
      //cout<<"\tCONTAINS NORX"<<endl;
    }
  }
  
  // Record the name of this sequence...
  mNames.push_back(Seq.mName);
}




#endif 





