
#ifndef MERREPEATMODEL
#define MERREPEATMODEL

#include "CBT.hh"
#include "Mer2IntMap.hh"

template<class MerHash>
class MerRepeatModel : public Mer2IntMap<MerHash> {
 public:

  string mName;

  int mMaxMerCount;
  int mMinMerCount;

  int mMinSavedCount;

  MerRepeatModel(){
    mMaxMerCount = 1000; 
    mMinMerCount = 2;    // Don't model singleton mers. 
    mMinSavedCount=1;
    mName = "MerModel";
  }

  bool isAcceptableFrequency(Mer &M){
    MerRepeatModel<MerHash> &MRM = (*this);

    // All mers acceptable if model is empty. 
    if (MRM.size() == 0) return(true);

    if (MRM.Contains(M)){
      int count = MRM[M];
      if ((count >= mMinMerCount) && (count <= mMaxMerCount)) return(true);
      else return(false);
    }else{
      // Mers not in model are assumed to be acceptable. 
      return(true);
    }
  }

  void setMaxCount(int maxc){mMaxMerCount = maxc;}
  void setMinCount(int minc){mMinMerCount = minc;}


  void WriteDist(string Name,int minOutputCount){
    ofstream fout;
    fout.open(Name.c_str());
    WriteDist(fout,minOutputCount);
    fout.close();
  }

  void WriteDist(ofstream &fout,int minOutputCount){
    typename Mer2IntMap<MerHash>::iterator it;

    cout<<"WriteDist mName = "<<mName<<endl;

    fout<<mName<<"\t"<<Mer::getMerSize()<<"\t"<<Mer::getThinFactor()<<"\t";
    fout<<Mer::getThinSlice()<<"\t"<<minOutputCount<<endl;
    for(it = (*this).begin();it != (*this).end();it++){
      if ((*it).second >= minOutputCount){
	string TempSeq = (*it).first.ToString();
	fout<<TempSeq<<"\t"<<(*it).second<<endl;
      }
    }
    fout.close();
  }

  void ReadDist(string FName,int minCount){
    izipstream zin;
    zin.open(FName.c_str());    
    ReadDist(zin,minCount);
    zin.close();
  }

  void ReadDist(istream &fin,int minCount)
  {
    // First line contains name of distribution, mer size, thin factor, and thin slice. 
    string line;
    GetLine(fin,line);
    
    int mersize, thinfactor,thinslice,offset=0;
    mName = SplitNext(line,offset);
    mersize = NextInt(line,offset);
    thinfactor = NextInt(line,offset);
    thinslice = NextInt(line,offset);
    mMinSavedCount = NextInt(line,offset);
    
    Mer::setMerSize(mersize);
    Mer::setThinFactor(thinfactor);
    Mer::setThinSlice(thinslice);  
    
    long merCount = 1,addCount = 0;
    do{
      fin.getline(readbuf,READ_BUFFER_SIZE);
      
      // Find the first whitespace
      int i;
      for(i = 0;i< READ_BUFFER_SIZE;i++){		
	if ((readbuf[i] == '\t') | (readbuf[i] == ' ')) break;
      }
      
      // Found a line starting with whitespace... end of the line. 
      if (i == 0) break;
      
      // Convert remainder of the line to an int. 
      int count = atoi(readbuf+i+1);		
      
      if (count >= minCount){		 
	Mer tempMer(readbuf);
	(*this)[tempMer] = count;
	addCount++;
      }
      if (!(merCount%100000)){
	//		cout<<readbuf<<endl;
	cerr<<"Mers seen:"<<merCount<<"\t size="<<(*this).size()<<endl;
      }
      merCount++;
    }while(!fin.eof());
  }


  // Adds the given distribution file to the current file...
  void AddDist(istream &fin,int minCount)
  {
    // First line contains name of distribution, mer size, thin factor, and thin slice. 
    string line;
    GetLine(fin,line);
    
    int mersize, thinfactor,thinslice,offset=0;
    string Name = SplitNext(line,offset);
    mersize = NextInt(line,offset);
    thinfactor = NextInt(line,offset);
    thinslice = NextInt(line,offset);
    int minsavedcount = NextInt(line,offset);
    
    //    Mer::setMerSize(mersize);
    //Mer::setThinFactor(thinfactor);
    //Mer::setThinSlice(thinslice);  

    if (Mer::getMerSize() != mersize) return;
    if (Mer::getThinFactor() != thinfactor) return;

    long merCount = 1,addCount = 0;
    do{
      fin.getline(readbuf,READ_BUFFER_SIZE);
      
      // Find the first whitespace
      int i;
      for(i = 0;i< READ_BUFFER_SIZE;i++){		
	if ((readbuf[i] == '\t') | (readbuf[i] == ' ')) break;
      }
      
      // Found a line starting with whitespace... end of the line. 
      if (i == 0) break;
      
      // Convert remainder of the line to an int. 
      int count = atoi(readbuf+i+1);		
      
      if (count >= minCount){		 
	Mer tempMer(readbuf);

	if ((*this).Contains(tempMer)){
	  (*this)[tempMer] += count;
	  addCount++;
	}else{
	  (*this)[tempMer] = count;
	  addCount++;
	}
      }
      if (!(merCount%100000)){
	//		cout<<readbuf<<endl;
	cout<<"Mers seen:"<<merCount<<"\t size="<<(*this).size()<<endl;
      }
      merCount++;
    }while(!fin.eof());
  }



};



#endif 


// Expected use. 
// if (mRepeatModel.merOK(TempMer)){
//   add(TempMer);
// }
