

class SequenceLocationBase {
 public: 
  
  virtual long sourceIdx() const =0;
  virtual long sourceStart()const =0;
  virtual bool isReverse()const =0;
  //virtual long sourceEnd()=0;
  
  virtual void setSourceIdx(long idx)=0;
  virtual void setSourceStart(long idx)=0;
  //  virtual void setSourceEnd(long idx)=0;
  virtual void setIsReverse()=0;
   
};

/*
SequenceLocation.hh:6: error:   virtual const long int 
   SequenceLocationBase::sourceIdx()
SequenceLocation.hh:7: error:   virtual const long int 
   SequenceLocationBase::sourceStart()
SequenceLocation.hh:8: error:   virtual const bool 
   SequenceLocationBase::isReverse()

*/

//----------------------------------------------------------------------------------------
// * BigSequenceLocation
//----------------------------------------------------------------------------------------
// Need another Hit for big sequences. 
// Ideally would like to be able to compare, say, 50Mbp pairwise. 
// 200Kbp vs. 20K (i.e. one BAC vs. all BACs).  
// 
// The most straightforward thing to do is to add a second int for 
// the BaseIdx.   
// 
// Listed from MSB to LSB: 
// 
// mData0:
// 1  bit   IsReverse.  Sense of the match 0 = fwd, 1 = reverse. 
// 19 bits  SourceRead (524,288 reads) 
// 12 bits  Unused.  (Just for now, to keep from having to rewrite SetSource, etc.) 
//
// mData1:
// 32 bits  BaseIdx
//
class BigSequenceLocation : SequenceLocationBase{
 public:
  unsigned int mData;
  unsigned int mData1;
  
  BigSequenceLocation(){
    mData=0;
    mData1=0;
  }

  //  void InitFrom(BigMerLocation &BM){
  //  mData = BM.mData;
  //  mData1 = BM.mData1;
  // }
  
  long sourceIdx() const {
    unsigned int sread = mData;
    sread = sread & 0x7FFFFFFF; // Mask out the IsReverse bit. 
    sread = sread >> 12;  // Shift out the BaseIdx, leaving the read.      
    return((long)sread);
  }

  long sourceStart() const {
    return((long)mData1);
  }

  //  int sourceEnd() const{
  //  return(mData1+Mer::mMerSize-1);
  // }
  void setSourceIdx(long SourceRead){
    unsigned int s = (unsigned int) SourceRead;
    s = s << 12;                       // Shift it over 12 bits.   
    unsigned int mask = 0x80000fff;    // Mask to protect the top bit and the lower 12 bits. 
    mData = mData & mask;              // clear the old
    mData = mData | s;                 // Set it to s
  }
  void setSourceStart(long BaseIdx){
    mData1 = BaseIdx;
  }  

  void setIsReverse(){
    //unsigned int mask = 0x7fffffff;   // Clear the old high bit.  
    //mData = mData & mask;
    mData = mData | 0x80000000;   // Set high bit. 
  }
  
  bool isReverse() const {
    unsigned int mask = 0x8fffffff;
    bool isrev = mData & 0x80000000;
    return(isrev);
  }
};








//========================== NOT YET CONVERTED ====================================
// Conversion of MerLocations to various kinds of Hits is ongoing. 

#if 0
//----------------------------------------------------------------------------------------
// * EmptyMerLocation
//----------------------------------------------------------------------------------------
// This is a mer location with no data part. 
// 
//
class EmptyMerLocation{
 public:
  int SourceRead() const {
  }  
  void SetSourceRead(int SourceRead){
    // There is always only one source read when this class is used. 
  }  
  int SourceBaseIdx() const {
  }  
  void SetBaseIdx(int BaseIdx){
  }  
  void SetIsReverse()
  {    //unsigned int mask = 0x7fffffff;   // Clear the old high bit.  
  }  
  bool IsReverse(){
  }
};



//----------------------------------------------------------------------------------------
// * SingleSourceMerLocation
//----------------------------------------------------------------------------------------
// This is for big sequences where there is guaranteed to only be one source
// sequence.  For example, the dot plotting program MegaDot plots a single source
// against multiple queries.  For this program there is no need to save the index 
// of the source that is used.  By eliminating the SourceIdx, we can achieve greater 
// space efficiency.  
// 
// mData0:
// 1  bit   IsReverse.  Sense of the match 0 = fwd, 1 = reverse. 
// 31 bits  BaseIdx
//
//
class SingleSourceMerLocation{
 public:
  unsigned int mData;
  
  SingleSourceMerLocation(){
    mData=0;
  }

  void InitFrom(SingleSourceMerLocation &BM){
    mData = BM.mData;
  }
  
  int SourceRead() const {
    return(1);
  }
  
  void SetSourceRead(int SourceRead){
    // There is always only one source read when this class is used. 
  }
  
  int SourceBaseIdx() const {
    unsigned int mask = 0x7fffffff;   // Clear the high bit.  
    unsigned int baseIdx = mData & mask;
    return(baseIdx);
  }
  
  void SetBaseIdx(int BaseIdx){
    unsigned int mask = 0x7fffffff;   
    BaseIdx = BaseIdx & mask; // Make sure high bit is clear. 
    mask = 0x80000000;

    mData = mData & mask; // Clear all but the high bit. 
    mData = mData | BaseIdx;
  }
  
  void SetIsReverse()
  {    //unsigned int mask = 0x7fffffff;   // Clear the old high bit.  
    //mData = mData & mask;
    mData = mData | 0x80000000;   // Set high bit. 
  }
  
  bool IsReverse(){
    unsigned int mask = 0x8fffffff;
    bool isrev = mData & 0x80000000;
    return(isrev);
  }
};




//----------------------------------------------------------------------------------------
// * MerLocation
//----------------------------------------------------------------------------------------
// A MerLocation is stored in the hash for each occurence of a mer in the sequence.  It 
// packs the information into a single int for space efficiency.  The SourceRead and the 
// BaseIdx of the Mer are encoded in an int as follows:
//
// Listed from MSB to LSB: 
// 
// 1  bit   IsReverse.  Sense of the match 0 = fwd, 1 = reverse. 
// 19 bits  SourceRead (524,288 reads) 
// 12 bits  BaseIdx    (4096 base positions)
// 
// Realistically, 16 bits, or 65K reads, is probably the most we can expect from a 1GB 
// machine, so 4 bits of the int could be used for other info if the need arises. 
//
class MerLocation{
 public:
  unsigned int mData;

  MerLocation(){
    mData=0;
  }

  void InitFrom(MerLocation &M){
    mData = M.mData;
  }

  int SourceRead() const {
    unsigned int sread = mData;
    sread = sread & 0x7FFFFFFF; // Mask out the IsReverse bit. 
    sread = sread >> 12;  // Shift out the BaseIdx, leaving the read.  

    return((int)sread);
  }

  void SetSourceRead(int SourceRead){
    unsigned int s = (unsigned int) SourceRead;
    s = s << 12;                       // Shift it over 12 bits.   
    unsigned int mask = 0x80000fff;    // Mask to protect the top bit and the lower 12 bits. 
    mData = mData & mask;              // clear the old
    mData = mData | s;                 // Set it to s
  }

  int SourceBaseIdx() const {
    unsigned int mask = 0x00000fff; 
    unsigned int bidx = mData;
    bidx = bidx & mask; // Mask out everything but lowest 12 bits. 
    return((int)bidx);
  }

  void SetBaseIdx(int BaseIdx){
    unsigned int b = BaseIdx;
    unsigned int mask = 0xfffff000;
    mData = mData & mask;              // clear the old
    mData = mData | b;
  }

  void SetIsReverse(){
    //unsigned int mask = 0x7fffffff;   // Clear the old high bit.  
    //mData = mData & mask;
    mData = mData | 0x80000000;   // Set high bit. 
  }
  
  bool IsReverse(){
    unsigned int mask = 0x8fffffff;
    bool isrev = mData & 0x80000000;
    return(isrev);
  }
};

#endif 
