#include "BandedAligner.hh"


int gBandCount = 0;

//-------------------------------------------
// * ComputeScore
//-------------------------------------------
// Computes banded alignment score with a traceback table from which 
// we can reconstruct the alignment. 
// 
int
BandedAligner::ComputeScore()
{
  Sequence &Query = (*pQuery);  // sugar. 
  Sequence &Source = (*pSource);
  int score = 0;

  gBandCount++;

  // A banded alignment looks at only a subset of the alignment table 
  // for performance.  In the code that follows, the prefix bt stands for 
  // band table.  All variables with bt prefix are in banded alignment table 
  // coordinates.   There is an imaginary full alignment table, with it's own 
  // coordinates.  I call this the virtual table, and variables in that coordinate
  // system are prefixed with "virtual". 
  register int btcol,btrow;
  register int diagScore,leftScore,topScore;
  register int NumBandRows = 2*BandSize+1;
  register int lastBandRow =2*BandSize;
  register int querySize = QEnd-QStart+1;

  // Initialize gap scores...
  for(btcol = 1;btcol <= BandSize;btcol++){
    ScoreTable[BandSize - btcol][btcol] = mGapScore*btcol;
  }

  for(btrow = BandSize;btrow <= lastBandRow;btrow++){
    ScoreTable[btrow][0] = mGapScore*(btrow -BandSize);
  }
    
  register int rowStart = BandSize;
  for(btcol = 1; btcol <= querySize;btcol++){
    register int btcolm = btcol -1;

    for(btrow = rowStart; btrow < NumBandRows;btrow++){
      
      // There is no score in the uupper or lower band size triangle of the matrix
      // When we find ourselves there, just continue without doing anything. 
      if ((btrow+btcol) < BandSize) continue;
      if ((btrow+btcol-querySize) > BandSize) continue;

      register int virtualRow = btcol - BandSize + btrow;

      // For performance, since this is used several times, only want to do the add once.
      register int btrowm = btrow -1;
      register int btrowp = btrow +1;

      // Offset into sequence. 
      register int basecol = btcol+QStart-1;
      register int baserow = virtualRow+SStart-1;

      if (baserow < Source.size()){
	if (Query[basecol] == Source[baserow]){
	  diagScore = ScoreTable[btrow][btcolm]+mMatchScore;
	}else{
	  diagScore = ScoreTable[btrow][btcolm]+mMisMatchScore;
	}
		  
	if (btrow < lastBandRow) leftScore = ScoreTable[btrowp][btcolm]+mGapScore;
	else leftScore = OUTSIDE_BAND_SCORE; 
		  
	if (btrowm >= 0) topScore = ScoreTable[btrowm][btcol]+mGapScore;
	else topScore = OUTSIDE_BAND_SCORE;
		
	// Find the max of the scores and record.  
	if (diagScore >= leftScore){
	  if (diagScore >= topScore){
	    ScoreTable[btrow][btcol]=diagScore;
	    setFlag(DIAGFLAG,TraceBack[btrow][btcol]);
	  }else{
	    ScoreTable[btrow][btcol]=topScore;
	    setFlag(TOPFLAG,TraceBack[btrow][btcol]);
	  }
	}else{
	  if (leftScore > topScore){
	    ScoreTable[btrow][btcol] = leftScore;
	    setFlag(LEFTFLAG,TraceBack[btrow][btcol]);
	  }else{
	    ScoreTable[btrow][btcol] = topScore;
	    setFlag(TOPFLAG,TraceBack[btrow][btcol]);
	  }
	}
      }
    }	 
    rowStart--;
    if (rowStart < 0) rowStart = 0;
  }
  score = ScoreTable[BandSize][querySize];  
  return(score);
}



void
BandedAligner::GetAlignment(Sequence &AlignSource,Sequence &AlignQuery){
  Sequence &Query = (*pQuery);
  Sequence &Source = (*pSource);

  int querySize = QEnd - QStart + 1;
  int btcol=querySize;
  //  int btrow=2*BandSize;
  int btrow = BandSize;
  
  while( (btcol > 0) && (btrow > 0)){  
    int virtualRow = btcol - BandSize + btrow;
    int basecol = btcol+QStart-1;
    int baserow = virtualRow+SStart-1;

    if (isSet(DIAGFLAG,TraceBack[btrow][btcol])){
      AlignSource.push_back(Source[baserow]);
      AlignQuery.push_back(Query[basecol]);
      btcol--;
      //      btrow--;
    }else{
      if (isSet(TOPFLAG,TraceBack[btrow][btcol])){
	AlignSource.push_back(Source[baserow]);
	AlignQuery.push_back('-');
	btrow--;
      }else{
	if (isSet(LEFTFLAG,TraceBack[btrow][btcol])){
	  AlignSource.push_back('-');
	  AlignQuery.push_back(Query[basecol]);
	  btcol--;
	  btrow++;
	}else{
	  cerr<<"**NONE** btrow="<<btrow<<"\tbtcol="<<btcol<<endl;
	}
      }
    }
  }
  
  // Reverse, since we constructed a backward version.  
  AlignSource.Reverse();
  AlignQuery.Reverse();
}

