//==============================================================================
// CLibraryAdditions.cc
//----------------------------------------------------------------------------
// Functions that mirror C library functions. 
//
//
//------------------------------------------------------------------------------
// $Id: CLibraryAdditions.cc,v 1.1 2000/03/31 19:32:26 kdurbin Exp kdurbin $
//------------------------------------------------------------------------------
// $Log: CLibraryAdditions.cc,v $
// Revision 1.1  2000/03/31 19:32:26  kdurbin
// Initial revision
//
//   
//==============================================================================
//.......1.........2.........3.........4.........5.........6.........7.........8
//

#include "CLibraryAdditions.hh"

char readbuf[READ_BUFFER_SIZE];

const double mypow10[]={1,10,100,1000,10000,100000,1000000,10000000,100000000,
		      1000000000.0,10000000000.0,100000000000.0};

//---------------------------------------------------------------------------
// * itoa
//---------------------------------------------------------------------------
//
//
string itoa(int num)
{
  char buf[STRING_BUFFER_SIZE];
  sprintf(buf,"%d",num);
  string temp = buf;
  return(temp);
}

//---------------------------------------------------------------------------
// * ftoa
//---------------------------------------------------------------------------
//
string ftoa(float num)
{
  char buf[STRING_BUFFER_SIZE];
  sprintf(buf,"%f",num);
  string temp = buf;
  return(temp);
}


//---------------------------------------------------------------------------
// * ftoa
//---------------------------------------------------------------------------
//
string ftoa(float num,string fmt)
{
  char buf[STRING_BUFFER_SIZE];
  sprintf(buf,fmt.c_str(),num);
  string temp = buf;
  return(temp);
}


//---------------------------------------------------------------------------
// * itostring
//---------------------------------------------------------------------------
//
//
string itostring(int num)
{
  char buf[STRING_BUFFER_SIZE];
  sprintf(buf,"%d",num);
  string temp = buf;
  return(temp);
}

//---------------------------------------------------------------------------
// * ftostring
//---------------------------------------------------------------------------
// Ditto.   
//
string ftostring(float num)
{
  char buf[STRING_BUFFER_SIZE];
  sprintf(buf,"%f",num);
  string temp = buf;
  return(temp);
}

//---------------------------------------------------------------------------
// * atof
//---------------------------------------------------------------------------
//
float atof(string &s)
{
  float val = atof(s.c_str());
  return(val);
}

//---------------------------------------------------------------------------
// * atoi
//---------------------------------------------------------------------------
//
int atoi(string &s)
{
  int val = atoi(s.c_str());
  return(val);
}


int todigit(char c){
  switch(c){
  case '0':return(0);
  case '1':return(1);
  case '2':return(2);
  case '3':return(3);
  case '4':return(4);
  case '5':return(5);
  case '6':return(6);
  case '7':return(7);
  case '8':return(8);
  case '9':return(9);
  default: cerr<<c<<"\t is not a digit"<<endl;
  }
}


//---------------------------------------------------------------------------
// * atou64
//---------------------------------------------------------------------------
//
uint64 atou64(string &s)
{
  uint64 val=0;

  int ssize = s.size();
  for(int i = ssize-1;i >= 0;i--){
    int digit = todigit(s[i]);
    double factor = digit*mypow10[ssize-i-1];
    val+=(uint64) factor;
  }
  return(val);
}


//---------------------------------------------------------------------------
// * sprintf
//---------------------------------------------------------------------------
// String version of sprintf
//
void sprintf(int MaxSize,string &outString,const char*fmt,...)
{
  va_list argp;
  
  // Maybe someday I'll write a version that figures out the size 
  // and expands the buffer automatically.
  int fmtSize = strlen(fmt);
  char *buf = new char[MaxSize];

  // Tell va mechanism what the last -named- argument is
  va_start(argp,fmt);
  vsprintf(buf,fmt,argp);
  outString = buf;
  delete buf;
}

//---------------------------------------------------------------------------
// * GetLine
//---------------------------------------------------------------------------
// getline is supposed to be builtin C++ standard library stuff, but some
// compilers are so lame that they don't implement this correctly. Obviously
// this is limited by READ_BUFFER_SIZE, which is set to something like 64K. 
//
bool GetLine(istream &theFile,string &theLine)
{
  bool status = theFile.getline(readbuf,READ_BUFFER_SIZE);
  theLine = readbuf;
  return(status);
}





