//========================================================================================
// Mer.cc
// ----------------------------------------------------------------------------
// $Id: //CBT-depot/CBT++/src/Mer.cc#16 $
// ------------------------------------------------------------------------------
// $Log: $
//
//========================================================================================
//.......1.........2.........3.........4.........5.........6.........7.........8.........9
//
#include "Mer.hh"

int Mer::mMerSize = 32; // Default mer size is 32 
int Mer::BASESUNUSED; // Default is 32-mer. 
int Mer::BITSUNUSED;
int Mer::mThinFactor = 1;
int Mer::mThinSlice = 1;



//----------------------------------------------------------------------------------------
// * InitWithFirstCleanMer
//----------------------------------------------------------------------------------------
//
// Initializes Mer with the first clean (no N or X) mer found after
// merStart. Returns the location where the next shift in should occurr. 
void 
Mer::InitWithFirstCleanMer(Sequence &S,int &merStart){
  // Shift in the first clean mer worth of bases...
  int mstart=merStart;
  int mend=mstart+Mer::getMerSize();
  
  for(int i = mstart;i < mend;i++){
    if (!Mer::isNX(S[i])){
      ShiftInBase(S[i]);
    }else{
      // If we find an N or X, want to jump ahead and try again
      // at the next possible clean mer, if possible. 
      //      i++;
      mend = i + Mer::getMerSize()+1;
      if (mend >= S.size()) break; // at end of sequence, abort. 
    }
  }
  merStart = mend-1;
}

//----------------------------------------------------------------------------------------
// * ShiftInClean
//----------------------------------------------------------------------------------------
// Ensures that M is efficiently set to the mer at merIdx or the 
// next clean (no N or X) mer after that if that mer isn't clean. a
//
// NOTE: This ONLY works in the case where the sequence is sampled 
// from the beginning. 
void 
Mer::ShiftInClean(Sequence &S,int &i){
  if (i == 0){
    InitWithFirstCleanMer(S,i);
    i = Mer::getMerSize()-1;
  }else{
    if (!Mer::isNX(S[i])){
      ShiftInBase(S[i]);
    }else{
      InitWithFirstCleanMer(S,i);
    }    
  }
}

//----------------------------------------------------------------------------------------
// * ShiftInBase
//----------------------------------------------------------------------------------------
//
void 
Mer::ShiftInBase(char b){
    switch(b){
      case 'A':
      case 'a':
	ShiftInBits(ACode); 
	break;
      case 'C':
      case 'c':
	ShiftInBits(CCode); 
	break;
      case 'G':
      case 'g':
	ShiftInBits(GCode); 
	break;
      case 'T':
      case 't':
	ShiftInBits(TCode); 
	break;
      default:
	//	cerr<<"INVALID BASE: "<<b<<endl;
	break;	  
    }
  }



//----------------------------------------------------------------------------------------
// * Mer
//----------------------------------------------------------------------------------------
// Constructor to create a 2-bit encoding of a specified subsequence of a sequence.   
// WARNING: Does not check that there is a valid mer at merStart offset in Seq!!!
//
Mer::Mer(Sequence &Seq,int merStart){
  mData = 0;
  int stop = merStart+mMerSize;
  for(register int i = merStart; i < stop; i++){
    char c = Seq[i];
    switch (c) {
      case 'a': case 'A': {
	ShiftInBits(ACode);
	break;
      }
      case 'c': case 'C': {
	ShiftInBits(CCode);
	break;
      }
      case 'g': case 'G': {
	ShiftInBits(GCode);
	break;
      }
      case 't': case 'T': {
	ShiftInBits(TCode);
	break;
      }
      default: {
	cerr<<"Mer(Seq,int) ILLEGAL CHARACTER: "<<c<<" from: "<<Seq<<endl; 
	//	return(0);
      }
    }
  }
}


//----------------------------------------------------------------------------------------
// * Mer
//----------------------------------------------------------------------------------------
// Constructor to create a 2-bit encoding of a specified subsequence of a sequence.   
//
Mer::Mer(Sequence &Seq,int merStart,bool &bContainsIllegal){
  mData = 0;
  bContainsIllegal = false;
  int stop = merStart+mMerSize;
  for(register int i = merStart; i < stop; i++){
    char c = Seq[i];
    switch (c) {
      case 'a': case 'A': {
	ShiftInBits(ACode);
	break;
      }
      case 'c': case 'C': {
	ShiftInBits(CCode);
	break;
      }
      case 'g': case 'G': {
	ShiftInBits(GCode);
	break;
      }
      case 't': case 'T': {
	ShiftInBits(TCode);
	break;
      }
      default: {
	bContainsIllegal = true;
      }
    }
  }
}



//----------------------------------------------------------------------------------------
// * Mer
//----------------------------------------------------------------------------------------
// Constructor to create a 2-bit encoding of a specified subsequence of a sequence.   
//
Mer::Mer(Sequence3Bit &Seq,int merStart,bool &bContainsIllegal){
  mData = 0;
  bContainsIllegal = false;
  int stop = merStart+mMerSize;
  for(register int i = merStart; i < stop; i++){
    char c = Seq[i];
    switch (c) {
      case 'a': case 'A': {
	ShiftInBits(ACode);
	break;
      }
      case 'c': case 'C': {
	ShiftInBits(CCode);
	break;
      }
      case 'g': case 'G': {
	ShiftInBits(GCode);
	break;
      }
      case 't': case 'T': {
	ShiftInBits(TCode);
	break;
      }
      default: {
	bContainsIllegal = true;
      }
    }
  }
}


//----------------------------------------------------------------------------------------
// * set
//----------------------------------------------------------------------------------------
//
void
Mer::set(const char* MerCStr)
{
  mData = 0;
  int stop = 32 - BASESUNUSED;
  for(register int i = 0; i < stop; i++){
    char c = MerCStr[i];
    switch (c) {
      case 'a': case 'A': {
	ShiftInBits(ACode);
	break;
      }
      case 'c': case 'C': {
	ShiftInBits(CCode);
	break;
      }
      case 'g': case 'G': {
	ShiftInBits(GCode);
	break;
      }
      case 't': case 'T': {
	ShiftInBits(TCode);
	break;
      }
      default: {
	//	cerr<<"Mer::set() ILLEGAL CHARACTER: "<<c<<" from: "<<MerCStr<<endl; 
	//	return(0);
      }
    }
  }
}


/*

//--------------------------------------
// * Reverse
//--------------------------------------
// Reverse complement a word.  
// 
inline void
Mer::Reverse() {

  register uint32 i;
  register uint64 rev = ~0;
  register uint64 mask = rev;
  register int msize = mMerSize;
   
  // cout<<hex<<endl;
  //cout<<"\t\t\tStarting data: "<<mData<<endl;
  //cout<<"\t\t\tBITSPERBASE="<<BITSPERBASE<<"\tBASEMASK="<<BASEMASK<<endl;

  //cout<<"\t\t\tStarting rev: "<<rev<<endl;

  // The word mData is treated as a string of 2-bit bases
  // (A == 01, C == 01, G == 10, T == 11, 
  // So long as we reverse the string basewise,
  // we just need to bit-complement to get the final answer.  
  for (i = 1; i <= msize; i++) {
    //  Shift result left and add in last base of input; then advance input
    rev = (rev << BITSPERBASE) + (mData & BASEMASK);
    mData >>= BITSPERBASE;
    //cout<<"\t\t\t"<<i<<"\trev="<<rev<<"\tmData="<<mData<<endl;
  }

  // Unfortunately, if we just take complement of this, 
  // it will set all the unused bits to 1.  This is fine so far
  // things like ToString() are concerned, but MerHash simply looks
  // at these as numbers, nothing else.  So, for MerHash 
  // ffffffffffcab06c  is not the same mer as 00000000000ab06c
  // even though, as 11 mers, these are the same.    
  mData = ~rev;

  //  cout<<"\t\t\tunmaskedrev: "<<mData<<endl;

  // So, need to explicitly set unused bits to zero... 
  mask = mask >> BITSUNUSED;
  //cout<<"\t\t\tmask="<<mask<<endl;
  mData = mData & mask;

  //cout<<"\t\t\tfinal mData="<<mData<<endl;
  //  cout<<dec<<endl;

}
*/


//--------------------------------------
// * ToString
//--------------------------------------
// 
string 
Mer::ToString() const {
  string temp;
  char base;
  uint32 basebits;
  uint64 w = mData;
  
  for(int i = 0;i < mMerSize;i++){
    basebits = w & BASEMASK;
    w >>= BITSPERBASE;
    switch (basebits) {
      case ACode: 
	base = 'A';
	break;
      case CCode: 
	base = 'C';
	break;
      case GCode: 
	base = 'G';
	break;
      case TCode: 
	base = 'T';
	break;
    }
    temp = base+temp;
  }
  return temp;
}

