#include "CBT.hh"
#include "MerHashMap.hh"

/* 
This format is called lav format, and is what is 
output by blastz.   An example follows:

#:lav
s {
  "RP31-78C13.fa" 1 119664 0 1
  "combined.new.fa" 1 2171922 0 1
}
h {
   ">RP31-78C13_AC091537.2 R norvegicus Brown Norway, complete sequence"
   ">CONTIG125_CONTIG134 "
}

a {
  s 5
  b 1 341803
  e 119664 461476
  l 1 341803 8927 350729 5	
}
#:eof


Each a{} grouping appears to be a single chain.  
s is the score, and seems to be ignored by laj. 
b is the beginning of the region of interest in source and query coords. 
e is the end of the region of interest in source and query coords. 
l is a line segment(??).  It gives beginning in source and query coords, 
  followed by the end in source and query coords, and finally a score.  
  This score, also, seems to be ignored by laj. 

Fwd and reverse hits are only distinguised by the direction the 
coordinates change. 

*/
class Segment{
public:
  long mScore;
  long mSBegin;
  long mSEnd;
  long mQBegin;
  long mQEnd;
};

// A chain is made up of a list of segments.  Each 
// segment is an span of exactly matching elements. 
// A chain may contain mismatches, gaps, or whatever, 
// but only between the segments.   The simplest kind of 
// segment is simply a sample.  
class Chain : public vector<Segment>{
public:  
  long mScore;
  int  mSourceIdx;
  long mSBegin;
  long mSEnd;
  long mQBegin;
  long mQEnd;

  float mAvgCount;
  int   totalCount;
};

class BlastzEmulator{
 public:

  BlastzEmulator(){
    f = 7;
    slice = 0;
    
  }

  //	SimpleMerHash mSMH;
  Mer2IntMap mSMH;
  
  MerHashMap<MerHashF, BigMerLocation> mSE;
  //MerHashMap<MerHashF, SingleSourceMerLocation> mSE;
  
  string mSourceName;
  string mQueryName;
  int   mQuerySkip;
  int   mSourceSkip;
  int   mWinLength;
  int   mWinWidth;
  int   mMinScore;
  int   mX0;
  int   mY0;
  bool  mbTabbedOut;
  long long f;
  long long slice;
  
  inline bool SampleInWindow(int x1,int y1, int x2, int y2, int WinWidth, int WinLength);
  inline bool SampleInRevWindow(int x1,int y1, int x2, int y2, int WinWidth, int WinLength);
  
  void AddSource(string SourceName);
  void AddSourceThinned(string SourceName);
  void Compare(istream &fin,ofstream &fout);
  void CompareThinned(istream &fin,ofstream &fout);
  
  void Compare(Sequence &Query,
	       MerSearchResultVec<BigMerLocation> &FwdHits,
	       MerSearchResultVec<BigMerLocation> &RevHits);
    
  void CompareThinned(Sequence &Query,
		      MerSearchResultVec<BigMerLocation> &FwdHits,
		      MerSearchResultVec<BigMerLocation> &RevHits);
    
  void ComputeChains(MerSearchResultVec<BigMerLocation> &Hits,
		     vector<Chain> &Chains);
  
  void ComputeRevChains(MerSearchResultVec<BigMerLocation> &Hits,
			vector<Chain> &Chains);
    
  void ComputeChains(Sequence &Query,MerSearchResultVec<BigMerLocation> &Hits,
		     ofstream &fout);
  
  void ComputeRevChains(Sequence &Query,MerSearchResultVec<BigMerLocation> &Hits,
			ofstream &fout);
  
  void WriteTabbedHeader(ostream &fout);
  void WriteTabbed(ostream &fout,Sequence &Query,Chain &theChain);

  void WriteLV(ostream &fout,Sequence &Query,Chain &X);
  void WriteLV(ostream &fout,Sequence &Query,vector<Chain> &Chains);

};



