#include "CBT.hh"
#include "BangEngine.hh"


float myFn(){
  cout<<"hi"<<endl;
}

// Note: I have emacs macros to automatically quote and unquote this region. 
// I find this the easiest way to edit help text. 
string gBangHelpString = 
    "\n"
    "\n"
    "\n  bang  {options} sourceFile queryFile OutputFile"
    "\n"
    "\n    Options:  "
    "\n"
    "\n    -M MinScore          [500]          Minimum score to report a chain."
    "\n    -s SampleSize        [32]           Sample size to use.  1-32.  8-32 practical. [32]"
    "\n"
    "\n    -F Thinning factor f                Only mers which satisfy  ((Mer /% f) == t) are used."
    "\n    -T Thinning slice  t                Only mers which satisfy  ((Mer /% f) == t) are used."
    "\n"
    "\n    -K K-mer distribution               File listing k-mer distribution. Overrides -s, -F,  and -T options "
    "\n                                        which are obtained from the kmner file."
    "\n    -k K-mer cutoff                     K-mers with counts > K-mer cutoff are ignored. " 
    "\n"
    "\n    -W WindowWidth       [10]           A band +/- WindowWidth will be considered for chaining. "
    "\n    -L WindowLength      [100]          Length of band to consider for chaining. "
    "\n"
    "\n"
    "\n    Performs a k-mer comparison of the source and query files, joining hits together along diagonals "
    "\n    in order to prodce chains in.  The density of source and query samples can be varied to reduce RAM "
    "\n    footprint or running time respectively (with associated loss of sensitivity). "
    "\n"
    "\n    The k-mers may be thinned so that only a subset of the k-mer space is examined.  That is, the "
    "\n    k-mer space itself may be reduced."
    "\n"
    "\n    1.2alpha         First fully function version to use Paul's mer encoding. "
    "\n    1.21alpha        Added Mer::ShiftInClean which gives 50% speed up on read processing. "
    "\n    1.24b            Added +/- field for fwd/reverse and made source coordinates reverse on reverse hit. "
"\n    1.26             Fixed several bugs in mer encoding. ";


void PrintHelp(){
  cout<<gBangHelpString<<endl;
}

// I find that I like to simply edit the help text, keeping a version history there, and not
// have to set an explicit version variable or anything.  As a result, I'm starting to make 
// this function a part of my boiler plate so version can be extracted from help. 
// 
string GetCurrentVersion(){
  int vidx = gBangHelpString.find("Current Version:");
  vidx = gBangHelpString.find(":",vidx)+1;
  return(SplitNext(gBangHelpString,vidx));
}

string gAllOptions = "";

int OptWinWidth = 10;
int OptWinLength= 100;

int OptMerSize = 32;

int OptMinScore = 500;

string OptSourceName = "";
string OptQueryName = "";
string OptOutName = "";

int  OptThinnedSlice = 1;
int  OptThinnedFactor = 1;

string OptKMerFile = "";
int    OptMaxKMerCount = 100;

bool   OptPerformAlignments = false;
bool   OptDebug = false;

//---------------------------------------------------
// * SetupOptions
//---------------------------------------------------
//
void SetupOptions(int argc, char**argv)
{
  // Print it for the record
  gAllOptions = "";
  for(int i = 0;i< argc;i++) {
    cout<<argv[i]<<" ";
    gAllOptions += argv[i];
    gAllOptions +="\t";
  }
  cout<<endl;

  float bob = myFn();
	
  // Create parser object and define valid options
  // Options with a : require a parameter. 
  GetOption Parser(argc,argv,"DAM:s:F:T:K:k:W:L:");
	
  // Handle the options...
  char theOption;
  while((theOption = Parser.NextOption()) != OPTION_EOF){
    switch(theOption){
      case 'M': OptMinScore =  Parser.GetIntArg();break;

      case 'W': OptWinWidth =  Parser.GetIntArg();break;
      case 'L': OptWinLength = Parser.GetIntArg();break;

      case 's': OptMerSize = Parser.GetIntArg();
	break;
      case 'T': OptThinnedSlice = Parser.GetIntArg();break;
      case 'F': OptThinnedFactor = Parser.GetIntArg();break;

      case 'K': OptKMerFile = Parser.GetStringArg();break;
      case 'k': OptMaxKMerCount = Parser.GetIntArg();break;

	// Experimental option.  
      case 'A': OptPerformAlignments = true;break;

    case 'D': OptDebug = true;break;

      default: PrintHelp();exit(1);
    }
  }

  OptSourceName = Parser.GetFreeArg();
  OptQueryName = Parser.GetFreeArg();
  OptOutName = Parser.GetFreeArg();

  if ((OptSourceName=="") || (OptQueryName=="") || (OptOutName=="")){
    PrintHelp();
    exit(1);
  }
}

int main(int argc,char** argv)
{
  cout<<"BANG"<<endl;
  SetupOptions(argc,argv);

  BangEngine B;
  B.mbDebug = OptDebug;
  B.mMinScore = OptMinScore;
  B.mbTabbedOut = true;      // Default now is tabbed output. 
  B.mbAlignments = OptPerformAlignments;
  

  // Reading the Mer distribution should automatically do this. 
  // If no mer distribution is given, set the size and thin factor. 
  if (OptKMerFile != ""){
    cout<<"Reading repeat model "<<OptKMerFile<<" ..."<<flush;
    B.SetupRepeatModel(OptKMerFile,OptMaxKMerCount);
    cout<<"done."<<endl;
    cout<<"MerSize: "<<Mer::getMerSize()<<"\tThinFactor: "<<Mer::getThinFactor()<<"\tThinSlice: "<<Mer::getThinSlice()<<endl;
  }else{
    Mer::setMerSize(OptMerSize);
    Mer::setThinFactor(OptThinnedFactor);
    Mer::setThinSlice(OptThinnedSlice);  
    
    cout<<"Size,Factor,Slice= "<<OptMerSize<<","<<OptThinnedFactor<<","<<OptThinnedSlice<<endl;
 
    // KJD So what happens when BangEngine calls Add(xxx,MerRepeatModel) ??  
    // Ans:  nothing.  An empty repeat model is declared in BangEngine, and 
    // that's passed in, and everything works as youl'd like. 
  }

  long startTime;
  StartTiming(startTime);
  cout<<"Adding "<<OptSourceName<<" to hash..."<<endl;
  B.AddSource(OptSourceName);
  double elapsed = EndTiming(startTime);
  cout<<elapsed<<" seconds. Hash size="<<B.mMerLocMap.size()<<endl;

  if (B.mbAlignments){
    cout<<"Alignments not yet implemented (but almost there...)"<<endl;
    //    cout<<"Adding "<<OptSourceName<<" for alignments..."<<endl;
    //B.ReadSourceForAlignments(OptSourceName);
    //cout<<"done."<<endl;
  }
  
  // For RAM efficiency, want to write results out as we 
  // generate them.  
  // 
  //  ifstream fin;
  izipstream fin;
  fin.open(OptQueryName.c_str());
  ofstream fout;
  fout.open(OptOutName.c_str());

  // Save command line options. 
  fout<<gAllOptions<<"\tVersion: "<<GetCurrentVersion()<<endl;

  StartTiming(startTime);
  cout<<"Comparing query "<<OptQueryName<<"..."<<flush;
  B.Compare(fin,fout);
  cout<<"done."<<endl;
  elapsed = EndTiming(startTime);
  cout<<elapsed<<" seconds."<<endl;

  fin.close();
  fout.close();  
  exit(0);
}
